@@ -6,15 +6,15 @@
  * Authors: Sivaiah Nallagatla <snallagatla@novell.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
@@ -143,12 +143,8 @@ e_book_backend_cache_constructor (GType type,
 	GObject *obj;
 	const char *uri;
 	char *cache_file;
-	EBookBackendCacheClass *klass;
-	GObjectClass *parent_class;
 
 	/* Invoke parent constructor. */
-	klass = E_BOOK_BACKEND_CACHE_CLASS (g_type_class_peek (E_TYPE_BOOK_BACKEND_CACHE));
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
 	obj = parent_class->constructor (type,
 					 n_construct_properties,
 					 construct_properties);
@@ -219,13 +215,13 @@ e_book_backend_cache_get_type (void)
 }
 
 /**
- * e_cal_backend_cache_new
+ * e_book_backend_cache_new
  * @uri: URI of the backend to be cached.
  *
- * Creates a new #EBookBackendCache object, which implements a cache of
- * calendar/tasks objects, very useful for remote backends.
+ * Creates a new #EBookBackendCache object, which implements a local
+ * cache of #EContact objects, useful for remote backends.
  *
- * Return value: The newly created object.
+ * Return value: A new #EBookBackendCache.
  */
 EBookBackendCache *
 e_book_backend_cache_new (const char *uri)
@@ -237,7 +233,17 @@ e_book_backend_cache_new (const char *uri)
         return cache;
 }
 
-
+/**
+ * e_book_backend_cache_get_contact:
+ * @cache: an #EBookBackendCache
+ * @uid: a unique contact ID
+ *
+ * Get a cached contact. Note that the returned #EContact will be
+ * newly created, and must be unreffed by the caller when no longer
+ * needed.
+ *
+ * Return value: A cached #EContact, or %NULL if @uid is not cached.
+ **/
 EContact *
 e_book_backend_cache_get_contact (EBookBackendCache *cache, const char *uid)
 {
@@ -257,7 +263,15 @@ e_book_backend_cache_get_contact (EBookBackendCache *cache, const char *uid)
 	return contact;
 }
 
-
+/**
+ * e_book_backend_cache_add_contact:
+ * @cache: an #EBookBackendCache
+ * @contact: an #EContact
+ *
+ * Adds @contact to @cache.
+ *
+ * Return value: %TRUE if the contact was cached successfully, %FALSE otherwise.
+ **/
 gboolean
 e_book_backend_cache_add_contact (EBookBackendCache *cache,
 				   EContact *contact)
@@ -285,7 +299,15 @@ e_book_backend_cache_add_contact (EBookBackendCache *cache,
 	return retval;
 }
 
-
+/**
+ * e_book_backend_cache_remove_contact:
+ * @cache: an #EBookBackendCache
+ * @uid: a unique contact ID
+ *
+ * Removes the contact identified by @uid from @cache.
+ *
+ * Return value: %TRUE if the contact was found and removed, %FALSE otherwise.
+ **/
 gboolean
 e_book_backend_cache_remove_contact (EBookBackendCache *cache,
 				    const char *uid)
@@ -309,6 +331,16 @@ e_book_backend_cache_remove_contact (EBookBackendCache *cache,
 
 	return retval;
 }
+
+/**
+ * e_book_backend_cache_check_contact:
+ * @cache: an #EBookBackendCache
+ * @uid: a unique contact ID
+ *
+ * Checks if the contact identified by @uid exists in @cache.
+ *
+ * Return value: %TRUE if the cache contains the contact, %FALSE otherwise.
+ **/
 gboolean 
 e_book_backend_cache_check_contact (EBookBackendCache *cache, const char *uid)
 {
@@ -327,11 +359,22 @@ e_book_backend_cache_check_contact (EBookBackendCache *cache, const char *uid)
 	return retval;
 }
 
+/**
+ * e_book_backend_cache_get_contacts:
+ * @cache: an #EBookBackendCache
+ * @query: an s-expression
+ *
+ * Returns a list of #EContact elements from @cache matching @query.
+ * When done with the list, the caller must unref the contacts and
+ * free the list.
+ *
+ * Return value: A #GList of pointers to #EContact.
+ **/
 GList *
 e_book_backend_cache_get_contacts (EBookBackendCache *cache, const char *query)
 {
         char *vcard_str;
-        GSList *l;
+        GSList *l, *lcache;
 	GList *list = NULL;
 	EContact *contact;
         EBookBackendSExp *sexp = NULL;
@@ -345,7 +388,7 @@ e_book_backend_cache_get_contacts (EBookBackendCache *cache, const char *query)
 	}
        
 
-        l = e_file_cache_get_objects (E_FILE_CACHE (cache));
+        lcache = l = e_file_cache_get_objects (E_FILE_CACHE (cache));
 
         for ( ; l != NULL; l = g_slist_next (l)) {
                 vcard_str = l->data;
@@ -353,18 +396,31 @@ e_book_backend_cache_get_contacts (EBookBackendCache *cache, const char *query)
                         contact = e_contact_new_from_vcard (vcard_str);
 			uid = e_contact_get_const (contact, E_CONTACT_UID);
                         if (contact && uid && *uid &&(query && e_book_backend_sexp_match_contact(sexp, contact)))
-				list = g_list_append (list, contact);
+				list = g_list_prepend (list, contact);
+			else
+				g_object_unref (contact);
                 }
                 
         }
-	if (l)
-		g_slist_free (l);
+	if (lcache)
+		g_slist_free (lcache);
 	if (sexp)
 		g_object_unref (sexp);
 
-        return list;
+        return g_list_reverse (list);
 }
 
+/**
+ * e_book_backend_cache_search:
+ * @cache: an #EBookBackendCache
+ * @query: an s-expression
+ *
+ * Returns an array of pointers to unique contact ID strings for contacts
+ * in @cache matching @query. When done with the array, the caller must
+ * free the ID strings and the array.
+ *
+ * Return value: A #GPtrArray of pointers to contact ID strings.
+ **/
 GPtrArray *
 e_book_backend_cache_search (EBookBackendCache *cache, const char *query)
 {
@@ -384,6 +440,14 @@ e_book_backend_cache_search (EBookBackendCache *cache, const char *query)
 	return ptr_array;
 }
 
+/**
+ * e_book_backend_cache_exists:
+ * @uri: URI for the cache
+ *
+ * Checks if an #EBookBackendCache exists at @uri.
+ *
+ * Return value: %TRUE if cache exists, %FALSE if not.
+ **/
 gboolean 
 e_book_backend_cache_exists (const char *uri)
 {
@@ -399,6 +463,13 @@ e_book_backend_cache_exists (const char *uri)
 	return exists;
 }
 
+/**
+ * e_book_backend_cache_set_populated:
+ * @cache: an #EBookBackendCache
+ *
+ * Flags @cache as being populated - that is, it is up-to-date on the 
+ * contents of the book it's caching.
+ **/
 void
 e_book_backend_cache_set_populated (EBookBackendCache *cache)
 {
@@ -407,11 +478,34 @@ e_book_backend_cache_set_populated (EBookBackendCache *cache)
 	
 }
 
+/**
+ * e_book_backend_cache_is_populated:
+ * @cache: an #EBookBackendCache
+ *
+ * Checks if @cache is populated.
+ *
+ * Return value: %TRUE if @cache is populated, %FALSE otherwise.
+ **/
 gboolean
 e_book_backend_cache_is_populated (EBookBackendCache *cache)
 {
-  	g_return_val_if_fail (E_IS_BOOK_BACKEND_CACHE (cache), NULL);
+  	g_return_val_if_fail (E_IS_BOOK_BACKEND_CACHE (cache), FALSE);
 	if (e_file_cache_get_object (E_FILE_CACHE (cache), "populated"))
 		return TRUE;
 	return FALSE;	
 }
+
+void
+e_book_backend_cache_set_time (EBookBackendCache *cache, const char *t)
+{
+	g_return_if_fail (E_IS_BOOK_BACKEND_CACHE (cache));
+	e_file_cache_add_object (E_FILE_CACHE (cache), "last_update_time", t);
+}
+
+char *
+e_book_backend_cache_get_time (EBookBackendCache *cache)
+{
+	g_return_val_if_fail (E_IS_BOOK_BACKEND_CACHE (cache), NULL);
+	return g_strdup (e_file_cache_get_object (E_FILE_CACHE (cache), "last_update_time"));
+}
+
