@@ -11,14 +11,12 @@
 #include <config.h>
 #endif
 
-#include <pthread.h>
 #include <string.h>
 
 #include <bonobo-activation/bonobo-activation.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-arg.h>
 #include "libedataserver/e-data-server-module.h"
-#include "e-data-book-marshal.h"
 #include "e-data-book-factory.h"
 
 #include <backends/groupwise/e-book-backend-groupwise.h>
@@ -43,6 +41,8 @@ struct _EDataBookFactoryPrivate {
 
 	/* Whether the factory has been registered with OAF yet */
 	guint       registered : 1;
+
+	int mode;
 };
 
 /* Signal IDs */
@@ -81,10 +81,11 @@ e_data_book_factory_extract_proto_from_uri (const char *uri)
 
 /**
  * e_data_book_factory_register_backend:
- * @factory:
- * @proto:
- * @backend:
- */
+ * @factory: an #EDataBookFactory
+ * @backend_factory: an #EBookBackendFactory
+ *
+ * Registers @backend_factory with @factory.
+ **/
 void
 e_data_book_factory_register_backend (EDataBookFactory      *book_factory,
 				      EBookBackendFactory   *backend_factory)
@@ -105,6 +106,17 @@ e_data_book_factory_register_backend (EDataBookFactory      *book_factory,
 			     g_strdup (proto), backend_factory);
 }
 
+static void
+out_of_proc_check (gpointer key, gpointer value, gpointer data)
+{
+	gboolean *out_of_proc = data;
+
+	if ((*out_of_proc))
+	    return;
+
+	*out_of_proc = e_book_backend_has_out_of_proc_clients (value);
+}
+
 /**
  * e_data_book_factory_get_n_backends:
  * @factory: An addressbook factory.
@@ -117,17 +129,30 @@ int
 e_data_book_factory_get_n_backends (EDataBookFactory *factory)
 {
 	int n_backends;
-
+	gboolean out_of_proc = FALSE;
+	
 	g_return_val_if_fail (factory != NULL, -1);
 	g_return_val_if_fail (E_IS_DATA_BOOK_FACTORY (factory), -1);
 
 	g_mutex_lock (factory->priv->map_mutex);
-	n_backends = g_hash_table_size (factory->priv->active_server_map);
+	g_hash_table_foreach (factory->priv->active_server_map, out_of_proc_check, &out_of_proc);
+
+	if (!out_of_proc)
+		n_backends = 0;
+	else
+		n_backends = g_hash_table_size (factory->priv->active_server_map);
 	g_mutex_unlock (factory->priv->map_mutex);
 
 	return n_backends;
 }
 
+/**
+ * e_data_book_factory_register_backends:
+ * @book_factory: an #EDataBookFactory
+ *
+ * Register the backends supported by the Evolution Data Server,
+ * with @book_factory.
+ **/
 void
 e_data_book_factory_register_backends (EDataBookFactory *book_factory)
 {
@@ -155,6 +180,13 @@ dump_active_server_map_entry (gpointer key, gpointer value, gpointer data)
 	g_message ("  %s: %p", uri, backend);
 }
 
+/**
+ * e_data_book_factory_dump_active_backends:
+ * @factory: an #EDataBookFactory
+ *
+ * Dump the list of active backends registered with @factory
+ * to stdout. This is a debugging function.
+ **/
 void
 e_data_book_factory_dump_active_backends (EDataBookFactory *factory)
 {
@@ -329,16 +361,12 @@ impl_GNOME_Evolution_Addressbook_BookFactory_getBook (PortableServer_Servant
 	g_free (uri);
 
 	if (backend) {
-		GNOME_Evolution_Addressbook_BookListener listener_copy;
-
-		listener_copy = bonobo_object_dup_ref (listener, NULL);
-
 		g_mutex_unlock (factory->priv->map_mutex);
 
 		book = e_data_book_new (backend, source, listener);
 
 		e_book_backend_add_client (backend, book);
-
+		e_book_backend_set_mode (backend, factory->priv->mode);
 		corba_book = bonobo_object_corba_objref (BONOBO_OBJECT (book));
 	}
 	else {
@@ -365,15 +393,24 @@ e_data_book_factory_construct (EDataBookFactory *factory)
 
 /**
  * e_data_book_factory_new:
- */
+ *
+ * Create a new #EDataBookFactory.
+ *
+ * Return value: A new #EDataBookFactory.
+ **/
 EDataBookFactory *
 e_data_book_factory_new (void)
 {
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+	static PortableServer_POA poa = NULL;
 	EDataBookFactory *factory;
 
-	factory = g_object_new (E_TYPE_DATA_BOOK_FACTORY, 
-				"poa", bonobo_poa_get_threaded (ORBIT_THREAD_HINT_PER_REQUEST, NULL),
-				NULL);
+	g_static_mutex_lock (&mutex);
+	if (poa == NULL)
+		poa = bonobo_poa_get_threaded (ORBIT_THREAD_HINT_PER_REQUEST, NULL);
+	g_static_mutex_unlock (&mutex);
+
+	factory = g_object_new (E_TYPE_DATA_BOOK_FACTORY, "poa", poa, NULL);
 	
 	e_data_book_factory_construct (factory);
 
@@ -382,7 +419,13 @@ e_data_book_factory_new (void)
 
 /**
  * e_data_book_factory_activate:
- */
+ * @factory: an #EDataBookFactory
+ * @iid: the OAF ID of the factory to activate
+ *
+ * Activates the factory specified by @iid, using Bonobo.
+ *
+ * Return value: %TRUE for success, %FALSE otherwise.
+ **/
 gboolean
 e_data_book_factory_activate (EDataBookFactory *factory, const char *iid)
 {
@@ -425,7 +468,36 @@ e_data_book_factory_activate (EDataBookFactory *factory, const char *iid)
 	g_free (tmp_iid);
 	return FALSE;
 }
+static void
+set_backend_online_status (gpointer key, gpointer value, gpointer data)
+{
+	EBookBackend *backend;
+	
+	backend = E_BOOK_BACKEND (value);
+	e_book_backend_set_mode (backend, GPOINTER_TO_INT (data));
+}
 
+/**
+ * e_data_book_factory_set_backend_mode:
+ * @factory: an #EDataBookFactory
+ * @mode: a connection status
+ *
+ * Sets all the backends associated with @factory to be either online
+ * or offline. @mode should be passed as 1 for offline, or 2 for
+ * online.
+ **/
+void
+e_data_book_factory_set_backend_mode (EDataBookFactory *factory, int mode)
+{
+	EDataBookFactoryPrivate *priv = factory->priv;
+	
+	
+	g_mutex_lock (priv->map_mutex);
+	priv->mode = mode;
+	g_hash_table_foreach (priv->active_server_map, set_backend_online_status, GINT_TO_POINTER (priv->mode));
+	g_mutex_unlock (priv->map_mutex);
+
+}
 static void
 e_data_book_factory_init (EDataBookFactory *factory)
 {
@@ -513,7 +585,7 @@ e_data_book_factory_class_init (EDataBookFactoryClass *klass)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (EDataBookFactoryClass, last_book_gone),
 			      NULL, NULL,
-			      e_data_book_marshal_NONE__NONE,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
 
