@@ -406,12 +406,12 @@ get_deltas (gpointer handle)
 
 	for (; item_list != NULL; item_list = g_list_next(item_list)) {
 		EGwItem *item = NULL;
-		item = E_GW_ITEM(item_list->data);
 		ECalComponent *modified_comp = NULL, *cache_comp = NULL;
 		char *cache_comp_str = NULL;
 		const char *uid, *rid = NULL;
 		int r_key;
 
+		item = E_GW_ITEM(item_list->data);
 		modified_comp = e_gw_item_to_cal_component (item, cbgw);
 		if (!modified_comp) {
 			g_message ("Invalid component returned in update");
@@ -1794,8 +1794,8 @@ sanitize_component (ECalBackendSync *backend, ECalComponent *comp, char *server_
 		icalproperty_set_x_name (icalprop, "X-GWRECORDID");
 		icalcomponent_add_property (e_cal_component_get_icalcomponent (comp), icalprop);
 
-		g_string_free (str, TRUE);
 	}
+	g_string_free (str, TRUE);
 }
 
 static ECalBackendSyncStatus
@@ -2250,15 +2250,9 @@ fetch_attachments (ECalBackendGroupwise *cbgw, ECalComponent *comp)
 	for (l = attach_list; l ; l = l->next) {
 		char *sfname = (char *)l->data;
 		char *filename, *new_filename;
-#if GLIB_CHECK_VERSION (2, 8, 0)
 		GMappedFile *mapped_file;
-#else
-		char *file_contents;
-		int len;
-#endif
 		GError *error = NULL;
 
-#if GLIB_CHECK_VERSION (2, 8, 0)
 		mapped_file = g_mapped_file_new (sfname, FALSE, &error);
 		if (!mapped_file) {
 			g_message ("DEBUG: could not map %s: %s\n",
@@ -2266,14 +2260,6 @@ fetch_attachments (ECalBackendGroupwise *cbgw, ECalComponent *comp)
 			g_error_free (error);
 			continue;
 		}
-#else
-		if (!g_file_get_contents (sfname, &file_contents, &len, &error)) {
-			g_message ("DEBUG: could not read %s: %s\n",
-				   sfname, error->message);
-			g_error_free (error);
-			continue;
-		}
-#endif
 		filename = g_path_get_basename (sfname);
 		new_filename = g_strconcat (uid, "-", filename, NULL);
 		g_free (filename);
@@ -2284,21 +2270,13 @@ fetch_attachments (ECalBackendGroupwise *cbgw, ECalComponent *comp)
 			/* TODO handle error conditions */
 			g_message ("DEBUG: could not open %s for writing\n",
 				   dest_file);
-#if GLIB_CHECK_VERSION (2, 8, 0)
 		} else if (write (fd, g_mapped_file_get_contents (mapped_file),
 				  g_mapped_file_get_length (mapped_file)) == -1) {
-#else
-		} else if (write (fd, file_contents, len) == -1) {
-#endif
 			/* TODO handle error condition */
 			g_message ("DEBUG: attachment write failed.\n");
 		}
 
-#if GLIB_CHECK_VERSION (2, 8, 0)
 		g_mapped_file_free (mapped_file);
-#else
-		g_free (file_contents);
-#endif
 		close (fd);
 		dest_url = g_filename_to_uri (dest_file, NULL, NULL);
 		g_free (dest_file);
@@ -2585,9 +2563,11 @@ send_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalcomp,
 					g_slist_free (comp_list);
 					g_free (old_object);
 				} else {
+					char * object;
+
 					id = e_cal_component_get_id (comp);
 					icalcomp = e_cal_component_get_icalcomponent (comp);
-					char * object = e_cal_component_get_as_string (comp);
+					object = e_cal_component_get_as_string (comp);
 				
 					if (e_cal_backend_cache_remove_component (priv->cache, id->uid, 
 								id->rid)) {
