@@ -346,7 +346,7 @@ camel_imap4_folder_new (CamelStore *store, const char *full_name, CamelException
 	
 	folder->summary = camel_imap4_summary_new (folder);
 	imap4_folder->cachedir = imap4_store_build_filename (store, folder->full_name);
-	e_util_mkdir_hier (imap4_folder->cachedir, 0777);
+	g_mkdir_with_parents (imap4_folder->cachedir, 0777);
 	
 	imap4_folder->cache = camel_data_cache_new (imap4_folder->cachedir, 0, NULL);
 	
@@ -996,7 +996,7 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 			resp = ic->resp_codes->pdata[i];
 			if (resp->code == CAMEL_IMAP4_RESP_CODE_APPENDUID) {
 				if (resp->v.appenduid.uidvalidity == summary->uidvalidity)
-					*appended_uid = g_strdup_printf ("%u", resp->v.appenduid.uid);
+					*appended_uid = g_strdup_printf ("%lu", resp->v.appenduid.uid);
 				break;
 			}
 		}
