@@ -38,7 +38,6 @@
 #include <libedataserverui/e-book-auth-util.h>
 #include <libedataserver/e-sexp.h>
 
-#include <libedataserverui/e-data-server-ui-marshal.h>
 #include "e-name-selector-entry.h"
 
 enum {
@@ -155,7 +154,7 @@ e_name_selector_entry_class_init (ENameSelectorEntryClass *name_selector_entry_c
 					 G_STRUCT_OFFSET (ENameSelectorEntryClass, updated),
 					 NULL,
 					 NULL,
-					 e_data_server_ui_marshal_VOID__POINTER,
+					 g_cclosure_marshal_VOID__POINTER,
 					 G_TYPE_NONE, 1, G_TYPE_POINTER);
 
 	g_type_class_add_private (object_class, sizeof(ENameSelectorEntryPrivate));
@@ -241,9 +240,6 @@ get_range_at_position (const gchar *string, gint pos, gint *start_pos, gint *end
 	gint         local_end_pos   = 0;
 	gint         i;
 
-	if (!string || !*string)
-		return FALSE;
-
 	for (p = string, i = 0; *p; p = g_utf8_next_char (p), i++) {
 		gunichar c = g_utf8_get_char (p);
 
@@ -1463,14 +1459,6 @@ user_focus_out (ENameSelectorEntry *name_selector_entry, GdkEventFocus *event_fo
 	if (!event_focus->in && priv->is_completing) {
 		entry_activate (name_selector_entry);
 	}
-	
-	if (name_selector_entry->type_ahead_complete_cb_id) {
-		g_source_remove (name_selector_entry->type_ahead_complete_cb_id);
-	}
-
-	if (name_selector_entry->update_completions_cb_id) {
-		g_source_remove (name_selector_entry->update_completions_cb_id);
-}
 
 	clear_completion_model (name_selector_entry);
 
@@ -2210,6 +2198,7 @@ e_name_selector_entry_init (ENameSelectorEntry *name_selector_entry)
 {
   GtkCellRenderer *renderer;
   ENameSelectorEntryPrivate *priv;
+  GConfClient *gconf;
 
   priv = E_NAME_SELECTOR_ENTRY_GET_PRIVATE (name_selector_entry);
 
@@ -2221,7 +2210,6 @@ e_name_selector_entry_init (ENameSelectorEntry *name_selector_entry)
   }
 
   /* read minimum_query_length from gconf*/
-  GConfClient *gconf;
   gconf = gconf_client_get_default();
   if (COMPLETION_CUE_MIN_LEN == 0) {
 	  if ((COMPLETION_CUE_MIN_LEN = gconf_client_get_int (gconf, MINIMUM_QUERY_LENGTH, NULL)))
